// package anonym_list -- список анонимных ключей в хранилище
package anonym_list

import (
	"fmt"
	"strings"

	"github.com/gofiber/fiber/v2"

	"git.p78su.freemyip.com/svi/gostore/pkg/types"
)

// AnonymList -- список анонимный ключей в хранилище
type AnonymList struct {
	serv  types.IService
	store types.IStore
}

// NewAnonymList -- возвращает список анонимных ключей в хранилище
func NewAnonymList(servHttp types.IServHttp, router fiber.Router) (*AnonymList, error) {
	{ // Предусловия
		if servHttp == nil {
			return nil, fmt.Errorf("NewAnonymList(): IServHttp==nil")
		}
		if router == nil {
			return nil, fmt.Errorf("NewAnonymList(): router==nil")
		}
	}
	sf := &AnonymList{
		serv:  servHttp.Service(),
		store: servHttp.Service().Store(),
	}
	router.Get("/list", sf.get)
	router.Post("/list", sf.post)
	return sf, nil
}

// Возвращает страницу анонимного раздела
func (sf *AnonymList) get(ctx *fiber.Ctx) error {
	return ctx.Render("anonym_list", fiber.Map{})
}

// ReadRequest -- читает значение из анонимного хранилища
type ReadRequest struct {
	Key_   string `json:"key" form:"key"` // Ключ для хранилища
	Value_ string `json:"val" form:"val"` // Значение для хранилища
}

const (
	strRow = `<div class="row">
	<div class="col-2">
		{{key}}
	</div>
	<div class="col-10">
		{{val}}
	</div>
	</div>
`
)

// Возвращает список ключей по префиксу из анонимного хранилища
func (sf *AnonymList) post(ctx *fiber.Ctx) error {
	req := &ReadRequest{}
	if err := ctx.BodyParser(req); err != nil {
		return ctx.Render("anonym_list", fiber.Map{
			"res": fmt.Sprintf("AnonymList.post(): при парсинге формы, err=%v", err),
		})
	}
	lstKey, err := sf.store.Find("anon/" + req.Key_)
	if err != nil {
		return ctx.Render("anonym_list", fiber.Map{
			"res": fmt.Sprintf("AnonymList.post(): при поиске анонимных ключей в хранилище, err=%v", err),
		})
	}
	strOut := ""
	for adr, val := range lstKey {
		binVal, err := sf.store.Get(val)
		if err != nil {
			binVal = []byte(fmt.Sprintf("AnonymList.post(): при поиске анонимного ключа %q в хранилище, err=%v", val, err))
		}
		req := &ReadRequest{
			Key_:   strings.TrimPrefix(val, "anon/"),
			Value_: string(binVal),
		}
		_row := strings.ReplaceAll(strRow, "{{key}}", req.Key_)
		_row = strings.ReplaceAll(_row, "{{val}}", req.Value_)
		strOut += _row
		if adr >= 50 {
			break
		}
	}
	strOut = `<div class="container-fluid border border-2 rounded-2">
<div class="row">
	<div class="col-2 border border-1 text-center">
			Ключ
	</div>
	<div class="col-10 border border-1 text-center">
		Значение
	</div>
</div>` + strOut + "</div><br>"
	return ctx.SendString(strOut)
}
